<?php

namespace eindwerk\Controller;

use eindwerk\Entity\User;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Container\ContainerInterface;


class UserController
{
    protected $container;
    protected $response;

    public function __construct(ContainerInterface $container)
    {
        $this->container = $container;
    }

    function validateUser($username, $pass)
    {
        $db = $this->container->get('db');
        $query = $db->prepare('
            SELECT name, password
            FROM user
            WHERE name = :name
        ');
        $query->bindValue(':name', $username);
        $res = $query->execute();
        $data = $res->fetchArray(SQLITE3_ASSOC);
        if ($data && password_verify($pass, $data['password'])) {
            return true;
        }
        return false;
    }

    function createToken($username)
    {
        $token = bin2hex(random_bytes(24));;
        $redis = $this->container->get('redis');
        if ($redis->set($token, $username)) {
            $redis->expire($token, 86400);
            return $token;
        } else {
            return false;
        }
    }

    public function validateToken(Request $request, Response $response): Response
    {
        $token = $request->getParsedBody()['token'];
        if (!$token) return $response->withStatus(401);

        $db = $this->container->get('db');
        $select = $db->prepare('
            SELECT name
            FROM user
            WHERE token = :token
        ');
        $select->bindValue(':token', $token);
        $res = $select->execute();
        $data = $res->fetchArray(SQLITE3_ASSOC);
        if ($data) {
            $delete = $db->prepare('
                UPDATE user
                SET token = NULL
                WHERE name = :name
            ');
            $delete->bindValue(':name', $data['name']);
            if ($delete->execute()) {
                $response->getBody()->write(json_encode($data));
                return $response
                    ->withHeader('Content-Type', 'application/json');
            }
        }
        return $response->withStatus(401);
    }

    public function resetPass(Request $request, Response $response): Response
    {
        $token = $request->getParsedBody()['token'];
        if (!$token) return $response->withStatus(401);

        $parsedBody = $request->getParsedBody();

        $hashed_pass = password_hash($parsedBody['password'], CRYPT_SHA256);

        $db = $this->container->get('db');
        $query = $db->prepare('
            UPDATE user
            SET password = :pass
            WHERE name = :token
        ');
        $query->bindValue(':token', $token);
        $query->bindValue(':pass', $hashed_pass);

        if ($query->execute()) {
            $response->getBody()->write('success');
        } else {
            $response->getBody()->write($db->lastErrorMsg());
            return $response->withStatus(403);
        }
        return $response;
    }

    public function resetPassMail(Request $request, Response $response): Response
    {

        $parsedBody = $request->getParsedBody();

        $db = $this->container->get('db');
        $token = '';
        $query = $db->prepare('
            SELECT token, name
            FROM user
            WHERE email = :email
        ');

        // TODO put constraints on username
        $query->bindValue(':email', $_POST['email']);
        $result = $query->execute();
        if ($query->execute()) {
            $response->getBody()->write(strval($db->lastInsertRowID()));
            while ($row = $result->fetchArray()) {
                $token = $row['token'];
                $name = $row['name'];
            }
            $mail = $this->container->get('mail');
            $mail->passwordReset($_POST['email'], $name);
        } else {
            $response->getBody()->write($db->lastErrorMsg());
            return $response->withStatus(403);
        }
        return $response;
    }

    function login(Request $request, Response $response, array $args)
    {
        $input = json_decode(file_get_contents('php://input'), true);
        if ($this->validateUser($input['username'], $input['password'])) {
            $data["accessToken"] = $this->createToken($input['username']);
        } else {
            return $response->withStatus(401);
        }
        $response->getBody()->write(json_encode($data));
        return $response
            ->withHeader('Content-Type', 'application/json');
    }

    function user(Request $request, Response $response, array $args)
    {
        $data["username"] = $request->getAttribute('username');

        $db = $this->container->get('db');

        $query = $db->prepare('
        SELECT E.id, E.user_name, E.first_name, E.last_name, E.email, E.address, E.rate, G.title
        FROM employee E
        LEFT JOIN grade G 
        ON E.grade_id = G.id
        WHERE user_name = :user 
        ');

        $query->bindValue(':user', $data["username"]);
        $res = $query->execute();

        if ($res) {
            $data = [];
            while ($row = $res->fetchArray(SQLITE3_ASSOC)) {
                array_push($data, $row);
            }
            $response->getBody()->write(json_encode($data));
            return $response->withHeader('Content-Type', 'application/json');
        } else {
            $response->getBody()->write($db->lastErrorMsg());
            return $response->withStatus(403);
        }
    }

    function editUser(Request $request, Response $response, array $args)
    {
        $parsedBody = $request->getParsedBody();

        $db = $this->container->get('db');

        $query = $db->prepare('
        UPDATE employee 
        SET first_name = :first_name, last_name = :last_name, email = :email, address = :address
        WHERE user_name = :user_name
        ');

        $query->bindValue(':user_name', $parsedBody['user_name']);
        $query->bindValue(':first_name', $parsedBody['first_name']);
        $query->bindValue(':last_name', $parsedBody['last_name']);
        $query->bindValue(':grade_id', $parsedBody['grade_id']);
        $query->bindValue(':email', $parsedBody['email']);
        $query->bindValue(':address', $parsedBody['address']);

        $res = $query->execute();

        if ($res) {
            $data = [];
            while ($row = $res->fetchArray(SQLITE3_ASSOC)) {
                array_push($data, $row);
            }
            $response->getBody()->write(json_encode($data));
            return $response->withHeader('Content-Type', 'application/json');
        } else {
            $response->getBody()->write($db->lastErrorMsg());
            return $response->withStatus(403);
        }
    }

    function logout(Request $request, Response $response, array $args)
    {
        $redis = $this->container->get('redis');
        $token = $request->getAttribute('token');
        $redis->del($token);
        return $response;
    }

    function create(Request $request, Response $response, array $args)
    {
        if (!isset($_POST['password']) || !isset($_POST['username']) || !isset($_POST['email'])) {
            $response->getBody()->write('Please fill out all fields.');
            return $response->withStatus(400);
        }

        $db = $this->container->get('db');
        $hashed_pass = password_hash($_POST['password'], CRYPT_SHA256);
        $token = bin2hex(random_bytes(24));;
        $query = $db->prepare('
            INSERT INTO user (name, password, token, email)
            VALUES (:name, :password, :token, :email);
        ');
        // TODO put constraints on username
        $query->bindValue(':name', $_POST['username']);
        $query->bindValue(':email', $_POST['email']);
        $query->bindValue(':password', $hashed_pass);
        $query->bindValue(':token', $token);

        if ($query->execute()) {
            $response->getBody()->write(strval($db->lastInsertRowID()));
            $mail = $this->container->get('mail');
            $mail->sendRegisterMail($_POST['email'], $_POST['username'], $token);
        } else {
            $response->getBody()->write($db->lastErrorMsg());
            return $response->withStatus(403);
        }

        $query = $db->prepare('
            INSERT INTO employee (user_name, grade_id, email)
            VALUES (:name, 1, :email);
        ');
        $query->bindValue(':name', $_POST['username']);
        $query->bindValue(':email', $_POST['email']);

        $query->execute();
        return $response;
    }

    public function getAllUsers(Request $request, Response $response, array $args): Response
    {
        $db = $this->container->get('db');
        $query = $db->prepare('
            SELECT E.id, E.user_name, E.first_name, E.last_name, E.email, E.address, E.grade_id
            FROM employee E
        ');
        $res = $query->execute();

        if ($res) {
            $data = [];
            while ($row = $res->fetchArray(SQLITE3_ASSOC)) {
                array_push($data, $row);
            }
            $response->getBody()->write(json_encode($data));
            return $response->withHeader('Content-Type', 'application/json');
        } else {
            $response->getBody()->write($db->lastErrorMsg());
            return $response->withStatus(403);
        }
    }
}
