<?php

use Dotenv\Dotenv;
use Predis\Client;
use Slim\Factory\AppFactory;
use DI\Container;
use Slim\Routing\RouteCollectorProxy;

use eindwerk\Service\DatabaseService;
use eindwerk\Service\MailerService;
use eindwerk\Controller\UserController;
use eindwerk\Controller\ProjectsController;
use eindwerk\Controller\ClientsController;
use eindwerk\Controller\TeamsController;
use eindwerk\Controller\InvoiceController;
use eindwerk\Middleware\AuthMiddleware;
use eindwerk\Middleware\CorsMiddleware;
use eindwerk\Middleware\JsonBodyParserMiddleware;
use eindwerk\Action\PreflightAction;

require __DIR__ . '/../vendor/autoload.php';

$dotenv = Dotenv::createImmutable(__DIR__ . '/../');
$dotenv->load();

/**
 * Initiate app
 **/
$container = new Container();
AppFactory::setContainer($container);

$container->set('db', function () {
    return new DatabaseService();
});

$container->set('redis', function () {
    return new Client($_ENV['REDIS']);
});

$container->set('mail', function () {
    return new MailerService(true);
});

$app = AppFactory::create();

/**
 * Routes
 **/

// Add CORS headers, needed when doing cross domain requests
$app->add(CorsMiddleware::class);

$app->group('/api', function (RouteCollectorProxy $group) {
    $group->post('/login', UserController::class . ':login');
    $group->options('/login', PreflightAction::class);

    $group->post('/create', UserController::class . ':create');
    $group->options('/create', PreflightAction::class);

    $group->post('/resetPass', UserController::class . ':resetPass');
    $group->options('/resetPass', PreflightAction::class);

    $group->post('/resetPassMail', UserController::class . ':resetPassMail');
    $group->options('/resetPassMail', PreflightAction::class);

    $group->post('/validate', UserController::class . ':validateToken')
        ->add(JsonBodyParserMiddleware::class);
    $group->options('/validate', PreflightAction::class);

    // OPTIONS requests don't contain Authorization header
    $group->options('/user', PreflightAction::class);
    $group->options('/logout', PreflightAction::class);
    $group->options('/user/projects', PreflightAction::class);
    $group->options('/projects', PreflightAction::class);
    $group->options('/project[/{id:[0-9]+}]', PreflightAction::class);
    $group->options('/tasks', PreflightAction::class);
    $group->options('/states', PreflightAction::class);
    $group->options('/mergeStates', PreflightAction::class);
    $group->options('/newTask', PreflightAction::class);
    $group->options('/taskUpdate', PreflightAction::class);
    $group->options('/BeginTaskUpdate', PreflightAction::class);
    $group->options('/saveTask', PreflightAction::class);
    $group->options('/assignUserToCard', PreflightAction::class);
    $group->options('/tasksGet', PreflightAction::class);
    $group->options('/deleteTask', PreflightAction::class);
    $group->options('/teams', PreflightAction::class);
    $group->options('/team[/{id:[0-9]+}]', PreflightAction::class);
    $group->options('/team/teamMembers', PreflightAction::class);
    $group->options('/user/AllUsers', PreflightAction::class);
    $group->options('/client[/{id:[0-9]+}]', PreflightAction::class);
    $group->options('/clients', PreflightAction::class);
    $group->options('/team/saveMembers', PreflightAction::class);
    $group->options('/profile[/{id:[0-9]+}]', PreflightAction::class);
    $group->options('/getInvoices', PreflightAction::class);
    $group->options('/updateInvoice', PreflightAction::class);
    $group->options('/getUserTasks[/{id:[0-9]+}]', PreflightAction::class);
});

$app->group('/api', function (RouteCollectorProxy $group) {
    $group->map(['GET', 'POST'], '/user', UserController::class . ':user');
    $group->post('/logout', UserController::class . ':logout');

    $group->post('/profile', UserController::class . ':editUser')
        ->add(JsonBodyParserMiddleware::class);
    $group->put('/profile/{id:[0-9]+}', UserController::class . ':editUser')
        ->add(JsonBodyParserMiddleware::class);

    $group->get('/user/projects', ProjectsController::class . ':getProjects');
    $group->get('/projects', ProjectsController::class . ':getAllProjects');
    $group->post('/project', ProjectsController::class . ':saveProject')
        ->add(JsonBodyParserMiddleware::class);
    $group->put('/project/{id:[0-9]+}', ProjectsController::class . ':saveProject')
        ->add(JsonBodyParserMiddleware::class);
    $group->get('/project/{id:[0-9]+}', ProjectsController::class . ':getProject');
    $group->get('/tasks', ProjectsController::class . ':getTasks');
    $group->get('/states', ProjectsController::class . ':getStates');
    $group->get('/mergeStates', ProjectsController::class . ':getMergeStates');
    $group->get('/getUserTasks/{id:[0-9]+}', ProjectsController::class . ':getUserTasks');

    $group->post('/newTask', ProjectsController::class . ':newTask');
    $group->post('/taskUpdate', ProjectsController::class . ':EndTaskUpdate');
    $group->post('/BeginTaskUpdate', ProjectsController::class . ':BeginTaskUpdate'); // still change these to a more better understandable link
    $group->post('/saveTask', ProjectsController::class . ':saveTask');
    $group->post('/assignUserToCard', ProjectsController::class . ':assignUserToCard');
    $group->post('/tasksGet', ProjectsController::class . ':getTasks');
    $group->post('/deleteTask', ProjectsController::class . ':deleteTask');

    $group->get('/teams', TeamsController::class . ':getAllTeams');
    $group->post('/team/saveMembers', TeamsController::class . ':saveTeamMembers');
    $group->get('/user/AllUsers', UserController::class . ':getAllUsers');
    $group->get('/team/teamMembers', TeamsController::class . ':getTeamMembers');
    $group->get('/clients', ClientsController::class . ':getAllClients');
    $group->post('/client', ClientsController::class . ':saveClient')
        ->add(JsonBodyParserMiddleware::class);
    $group->put('/client/{id:[0-9]+}', ClientsController::class . ':saveClient')
        ->add(JsonBodyParserMiddleware::class);
    $group->get('/client/{id:[0-9]+}', ClientsController::class . ':getClient');
    $group->post('/team', TeamsController::class . ':saveTeam')
        ->add(JsonBodyParserMiddleware::class);
    $group->put('/team/{id:[0-9]+}', TeamsController::class . ':saveTeam')
        ->add(JsonBodyParserMiddleware::class);
    $group->get('/team/{id:[0-9]+}', TeamsController::class . ':getTeam');


    $group->get('/getInvoices', InvoiceController::class . ':getAllInvoices');
    $group->post('/updateInvoice', InvoiceController::class . ':updateInvoice');
})->add(AuthMiddleware::class);

// Add Routing Middleware
$app->addRoutingMiddleware();

// Add Error Middleware
$app->addErrorMiddleware(true, true, true);

$app->run();
